// BoardWidget.cpp //

#include "BoardWidget.h"
#include <sstream>
#include <string>
#include "UI.h"

using std::stringstream;

BoardWidget :: BoardWidget (Game *poGame)
: Widget(c_kiYSize+2, c_kiXSize+(c_kiLWidth*2)+2, c_kiYPos, c_kiXPos, true),
m_poGame(NULL)
{
  keypad(m_poWin, true);
  m_poGame = poGame;
}

void BoardWidget :: Draw (void)
{
  /// if there's no board currently set
  if (NULL == m_poGame->GetBoard())
  {
    DrawNoBoard();
  }
  else
  {
    DrawBoard();
  }
}

int BoardWidget :: KeyFocus (void)
{
  curs_set(0);
  int iCh = GetCh(0, 0);
  switch (iCh)
  {
    case 'q':
      return iCh;
  }
  return iCh;
}


void BoardWidget :: DrawNoBoard (void)
{
  string oStrText = "No board loaded";
  AddStr(c_kiYSize/2, (c_kiXSize-oStrText.length())/2+3, oStrText);
  curs_set(0);
}


void BoardWidget :: DrawBoard (void)
{
  int iY = c_kiYSize - 2;    int iI = 1, iJ = 10;
  for(int i=0; i<20; i++)
  {
    stringstream oSrm;
    oSrm.width(c_kiLWidth-1);
    if(i % 2 == 0)
    {
      oSrm << iI;
      iI += 10;
    }
    else
    {
      oSrm << iJ;
      iJ += 10 ;
    }
    string ostr = oSrm.str();
    if(i % 2 == 0)
    {
      AddStr(iY, ((i/2)%2)?(c_kiXSize+c_kiLWidth):0, ostr);
    }
    else
    {
      AddStr(iY, ((i/2)%2)?0:(c_kiXSize+c_kiLWidth), ostr);
      iY = iY - c_kiCHeight;
    }
  }
  
  vector<Player> *poVecPlayers = m_poGame->GetPlayers();
  for(int i=0; i<100; i++)
  {
    int iRow = i/10;
    int iCol = i%10;
    int iY = c_kiYSize - iRow * c_kiCHeight - 1;
    int iX = ((iRow%2)?(c_kiXSize + c_kiLWidth - (iCol+1)*c_kiCWidth)
        :(iCol*c_kiCWidth + c_kiLWidth));
    
    int iAttri;
    int iEndPos = -1;
    if((*m_poGame->GetBoard())[i].GetType() == SNAKE)
    {
      iAttri = COLOR_PAIR(COLORPAIR_3)|WA_REVERSE;
      iEndPos = (*m_poGame->GetBoard())[i].GetEndPos();
    }
    else if((*m_poGame->GetBoard())[i].GetType() == LADDER)
    {
      iAttri = COLOR_PAIR(COLORPAIR_4)|WA_REVERSE;
      iEndPos = (*m_poGame->GetBoard())[i].GetEndPos();
    }
    else if(i % 2 == 0)
    {
      iAttri = COLOR_PAIR(COLORPAIR_2)|WA_REVERSE;
    }
    else
    {
      iAttri = COLOR_PAIR(COLORPAIR_1)|WA_REVERSE;
    }

    string ostr;
    bool bHasPlayer = false;
    for (int iPIdx=0; iPIdx < poVecPlayers->size(); iPIdx++)
    {
      if ((*poVecPlayers)[iPIdx].GetCurPos() == i)
      {
        ostr += (*poVecPlayers)[iPIdx].GetPID();
        bHasPlayer = true;
      }
    }
    if (iEndPos != -1)
    {
      while (ostr.size() < (2 * c_kiCWidth - 2))
      {
        ostr += ' ';
      }
      stringstream oSrmPos;
      oSrmPos.width(2);
      oSrmPos << iEndPos+1;
      ostr += oSrmPos.str();
    }
    else
    {
      while (ostr.size() < 2 * c_kiCWidth)
      {
        ostr += ' ';
      }
    }
    if (bHasPlayer)
    {
      wattron(m_poWin,iAttri | WA_BOLD);
      AddStr(iY,iX,ostr.substr(0, c_kiCWidth));        
      AddStr(iY-1,iX,ostr.substr(c_kiCWidth, c_kiCWidth));
      wattroff(m_poWin,iAttri | WA_BOLD);
    }
    else
    {
      wattron(m_poWin,iAttri);
      AddStr(iY,iX,ostr.substr(0, c_kiCWidth));        
      AddStr(iY-1,iX,ostr.substr(c_kiCWidth, c_kiCWidth));
      wattroff(m_poWin,iAttri);
    }
  }
  curs_set(0);
}

void BoardWidget::Edit(void)
{
  int iCursor = 0;
  int iFrom = -1;
  int iTo = -1;

  while (true)
  {
    DrawEdit(iCursor);
    Refresh();
    int iCh = GetCh(0,0);

    switch (iCh)
    {
      case 9:
        UI::Instance()->Status("Done editing");
        return;
        break;

      case KEY_DOWN:
      case KEY_LEFT:
        if (iCursor > 0)
        {
          iCursor--;
        }
        else
        {
          beep();
        }
        break;

      case KEY_UP:
      case KEY_RIGHT:
        if (iCursor < 99)
        {
          iCursor++;
        }
        else
        {
          beep();
        }
        break;

      case KEY_DC:
        try
        {
          m_poGame->GetBoard()->DelPortal(iCursor);
        }
        catch (runtime_error& rErr)
        {
          UI::Instance()->Status(rErr.what());
        }
        break;

      case 10:
        if (iFrom == -1)
        {
          iFrom = iCursor;
          stringstream oSrm;
          oSrm << (iFrom+1) << " -> ?";
          string oStr = oSrm.str();
          UI::Instance()->Status(oStr);
        }
        else if (iTo == -1)
        {
          iTo = iCursor;
          try 
          {
            m_poGame->GetBoard()->AddPortal(iFrom, iTo);
            stringstream oSrm;
            oSrm << (iFrom+1) << " -> " << (iTo+1);
            string oStr = oSrm.str();
            UI::Instance()->Status(oStr);
          }
          catch (runtime_error& rErr)
          {
            UI::Instance()->Status(rErr.what());
          }
          iFrom = -1;
          iTo = -1;
        }
        else
        {
          beep();
        }
        break;

      default:
        beep();
        break;
    }
  }
}

void BoardWidget::DrawEdit (int iCursor)
{
  int iY = c_kiYSize - 2;    int iI = 1, iJ = 10;
  for(int i=0; i<20; i++)
  {
    stringstream oSrm;
    oSrm.width(c_kiLWidth-1);
    if(i % 2 == 0)
    {
      oSrm << iI;
      iI += 10;
    }
    else
    {
      oSrm << iJ;
      iJ += 10 ;
    }
    string ostr = oSrm.str();
    if(i % 2 == 0)
    {
      AddStr(iY, ((i/2)%2)?(c_kiXSize+c_kiLWidth):0, ostr);
    }
    else
    {
      AddStr(iY, ((i/2)%2)?0:(c_kiXSize+c_kiLWidth), ostr);
      iY = iY - c_kiCHeight;
    }
  }
  
  vector<Player> *poVecPlayers = m_poGame->GetPlayers();
  for(int i=0; i<100; i++)
  {
    int iRow = i/10;
    int iCol = i%10;
    int iY = c_kiYSize - iRow * c_kiCHeight - 1;
    int iX = ((iRow%2)?(c_kiXSize + c_kiLWidth - (iCol+1)*c_kiCWidth)
        :(iCol*c_kiCWidth + c_kiLWidth));
    
    int iAttri;
    int iEndPos = -1;
    if((*m_poGame->GetBoard())[i].GetType() == SNAKE)
    {
      iAttri = COLOR_PAIR(COLORPAIR_3)|WA_REVERSE;
      iEndPos = (*m_poGame->GetBoard())[i].GetEndPos();
    }
    else if((*m_poGame->GetBoard())[i].GetType() == LADDER)
    {
      iAttri = COLOR_PAIR(COLORPAIR_4)|WA_REVERSE;
      iEndPos = (*m_poGame->GetBoard())[i].GetEndPos();
    }
    else if(i % 2 == 0)
    {
      iAttri = COLOR_PAIR(COLORPAIR_2)|WA_REVERSE;
    }
    else
    {
      iAttri = COLOR_PAIR(COLORPAIR_1)|WA_REVERSE;
    }

    string ostr;
    if (i == iCursor)
    {
      ostr = "##  ##";
    }
    if (iEndPos != -1)
    {
      while (ostr.size() < (2 * c_kiCWidth - 2))
      {
        ostr += ' ';
      }
      stringstream oSrmPos;
      oSrmPos.width(2);
      oSrmPos << iEndPos+1;
      ostr += oSrmPos.str();
    }
    while (ostr.size() < 2 * c_kiCWidth)
    {
      ostr += ' ';
    }
    wattron(m_poWin,iAttri);
    AddStr(iY,iX,ostr.substr(0, c_kiCWidth));        
    AddStr(iY-1,iX,ostr.substr(c_kiCWidth, c_kiCWidth));
    wattroff(m_poWin,iAttri);

  }
  curs_set(0);
}

// BoardWidget.cpp //
