// Widget.h //

#ifndef __WIDGET_H
#define __WIDGET_H

#include <sys/types.h>
#include <curses.h>
#include <stdexcept>
#include <panel.h>
#include "ui_ex.h"

using std::string;

#define COLORPAIR_1 1
#define COLORPAIR_2 2
#define COLORPAIR_3 3
#define COLORPAIR_4 4
#define COLORPAIR_5 5
#define COLORPAIR_6 6

/** @brief the widget class
 *
 * abstracts display elements in the application
 */
class Widget
{
  public:
    /// constructor takes coordinates, dimensions and properties
    Widget (int iYSize, int iXSize, int iYPosn, int iXPosn
        , bool bHasBorder=false);

    /// destructor
    ~Widget (void);

    /// render the element
    void Render (void);

    /// refresh the window corresponding to this widget
    void Refresh (void) { wrefresh(m_poWin); }

    /// application logic of the particular type of widget
    virtual void Draw (void)=0;

    /// this window gets the keyboard focus
    virtual int KeyFocus (void)=0;

    /// get the underlying WINDOW
    WINDOW *GetWin (void) { return m_poWin; }

    /// add a string to the widget
    void AddStr(int iYPos, int iXPos, const string& rkoStrText);

    /// draw a horizontal line
    void HLine(int iYPos);

    /// draw a horizontal line
    void VLine(int iXPos);

    /// get a character
    int GetCh (int iYPos, int iXPos);

    /// bring this window to the top
    void Top (void);

    /// hide this window
    void Hide (void);

    /// show this panel
    void Show (void);

  protected:
    /// Draw a Bitmap on the widget
    void DrawBitmap (const string& rkostrBitmap, int iAttri);

    /// the x dimension (limited to MAXX usually 80)
    int m_iXSize;

    /// the y dimension (limited to MAXY usually 24)
    int m_iYSize;

    /// the y position (limited to MAXY usually 24)
    int m_iYPosn;
    
    /// the x position (limited to MAXX usually 24)
    int m_iXPosn;

    /// the underlying window
    WINDOW *m_poWin;

    /// the panel containing the underlying window
    PANEL *m_poPanel;

    /// whether the window has a border
    bool m_bHasBorder;
    
  private:
};

#endif //__WIDGET_H

// Widget.h //
